@@ -24,6 +24,7 @@
 #include "gkd-dbus-util.h"
 
 #include "gkd-secret-error.h"
+#include "gkd-secret-introspect.h"
 #include "gkd-secret-objects.h"
 #include "gkd-secret-property.h"
 #include "gkd-secret-secret.h"
@@ -48,7 +49,6 @@ struct _GkdSecretObjects {
 	GObject parent;
 	GkdSecretService *service;
 	GckSlot *pkcs11_slot;
-	GHashTable *aliases;
 };
 
 G_DEFINE_TYPE (GkdSecretObjects, gkd_secret_objects, G_TYPE_OBJECT);
@@ -70,7 +70,7 @@ parse_object_path (GkdSecretObjects *self, const gchar *path, gchar **collection
 		return FALSE;
 
 	if (g_str_has_prefix (path, SECRET_ALIAS_PREFIX)) {
-		replace = g_hash_table_lookup (self->aliases, *collection);
+		replace = gkd_secret_service_get_alias (self->service, *collection);
 		if (!replace) {
 
 			/*
@@ -408,7 +408,7 @@ item_method_set_secret (GkdSecretObjects *self, GckObject *item, DBusMessage *me
 	GkdSecretSecret *secret;
 	const char *caller;
 
-	if (!dbus_message_has_signature (message, "(oayay)"))
+	if (!dbus_message_has_signature (message, "(oayays)"))
 		return NULL;
 	dbus_message_iter_init (message, &iter);
 	secret = gkd_secret_secret_parse (self->service, message, &iter, &derr);
@@ -455,7 +455,7 @@ item_message_handler (GkdSecretObjects *self, GckObject *object, DBusMessage *me
 		return item_property_getall (object, message);
 
 	else if (dbus_message_has_interface (message, DBUS_INTERFACE_INTROSPECTABLE))
-		return gkd_dbus_introspect_handle (message, "item");
+		return gkd_dbus_introspect_handle (message, gkd_secret_introspect_item);
 
 	return NULL;
 }
@@ -661,7 +661,7 @@ collection_method_create_item (GkdSecretObjects *self, GckObject *object, DBusMe
 	gboolean created = FALSE;
 
 	/* Parse the message */
-	if (!dbus_message_has_signature (message, "a{sv}(oayay)b"))
+	if (!dbus_message_has_signature (message, "a{sv}(oayays)b"))
 		return NULL;
 	if (!dbus_message_iter_init (message, &iter))
 		g_return_val_if_reached (NULL);
@@ -806,7 +806,7 @@ collection_message_handler (GkdSecretObjects *self, GckObject *object, DBusMessa
 		return collection_property_getall (self, object, message);
 
 	else if (dbus_message_has_interface (message, DBUS_INTERFACE_INTROSPECTABLE))
-		return gkd_dbus_introspect_handle (message, "collection");
+		return gkd_dbus_introspect_handle (message, gkd_secret_introspect_collection);
 
 	return NULL;
 }
@@ -830,7 +830,7 @@ gkd_secret_objects_constructor (GType type, guint n_props, GObjectConstructParam
 static void
 gkd_secret_objects_init (GkdSecretObjects *self)
 {
-	self->aliases = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
+
 }
 
 static void
@@ -857,7 +857,6 @@ gkd_secret_objects_finalize (GObject *obj)
 {
 	GkdSecretObjects *self = GKD_SECRET_OBJECTS (obj);
 
-	g_hash_table_destroy (self->aliases);
 	g_assert (!self->pkcs11_slot);
 	g_assert (!self->service);
 
@@ -1292,7 +1291,7 @@ gkd_secret_objects_handle_get_secrets (GkdSecretObjects *self, DBusMessage *mess
 
 	reply = dbus_message_new_method_return (message);
 	dbus_message_iter_init_append (reply, &iter);
-	dbus_message_iter_open_container (&iter, DBUS_TYPE_ARRAY, "{o(oayay)}", &array);
+	dbus_message_iter_open_container (&iter, DBUS_TYPE_ARRAY, "{o(oayays)}", &array);
 
 	for (i = 0; i < n_paths; ++i) {
 
@@ -1332,20 +1331,3 @@ gkd_secret_objects_handle_get_secrets (GkdSecretObjects *self, DBusMessage *mess
 
 	return reply;
 }
-
-const gchar*
-gkd_secret_objects_get_alias (GkdSecretObjects *self, const gchar *alias)
-{
-	g_return_val_if_fail (GKD_SECRET_IS_OBJECTS (self), NULL);
-	g_return_val_if_fail (alias, NULL);
-	return g_hash_table_lookup (self->aliases, alias);
-}
-
-void
-gkd_secret_objects_set_alias (GkdSecretObjects *self, const gchar *alias,
-                              const gchar *identifier)
-{
-	g_return_if_fail (GKD_SECRET_IS_OBJECTS (self));
-	g_return_if_fail (alias);
-	g_hash_table_replace (self->aliases, g_strdup (alias), g_strdup (identifier));
-}
