@@ -19,6 +19,10 @@
  * 02111-1307, USA.  
  */
 
+#if !defined (__GCR_H_INSIDE__) && !defined (GCR_COMPILATION)
+#error "Only <gcr/gcr.h> can be included directly."
+#endif
+
 #ifndef __GCR_IMPORTER_H__
 #define __GCR_IMPORTER_H__
 
@@ -53,38 +57,38 @@ struct _GcrImporter {
 
 struct _GcrImporterClass {
 	GObjectClass parent_class;
-	
+
 	/* signals */
-	
-	void (*imported) (GcrImporter *self, struct _GckObject *object);
+	void (*queued) (GcrImporter *self, const gchar *label, GckAttributes *attrs);
+	void (*imported) (GcrImporter *self, GckObject *object);
 };
 
 GType                     gcr_importer_get_type               (void);
 
 GcrImporter*              gcr_importer_new                    (void);
 
-GcrParser*                gcr_importer_get_parser             (GcrImporter *self);
-
-void                      gcr_importer_set_parser             (GcrImporter *self,
-                                                               GcrParser *parser);
-
-struct _GckSlot*          gcr_importer_get_slot               (GcrImporter *self);
+GckSlot*                  gcr_importer_get_slot               (GcrImporter *self);
 
 void                      gcr_importer_set_slot               (GcrImporter *self,
-                                                               struct _GckSlot *slot);
+                                                               GckSlot *slot);
 
 GcrImporterPromptBehavior gcr_importer_get_prompt_behavior    (GcrImporter *self);
 
 void                      gcr_importer_set_prompt_behavior    (GcrImporter *self,
                                                                GcrImporterPromptBehavior behavior);
 
+void                      gcr_importer_queue                  (GcrImporter *self,
+                                                               const gchar *label,
+                                                               GckAttributes *attrs);
+
+void                      gcr_importer_listen                 (GcrImporter *self,
+                                                               GcrParser *parser);
+
 gboolean                  gcr_importer_import                 (GcrImporter *self,
-                                                               GInputStream *input,
                                                                GCancellable *cancel,
                                                                GError **error);
 
 void                      gcr_importer_import_async           (GcrImporter *self,
-                                                               GInputStream *input,
                                                                GCancellable *cancel,
                                                                GAsyncReadyCallback callback,
                                                                gpointer user_data);
